#!/usr/bin/env ruby

##
# submit50 submission script
#
# Tommy MacWilliam <tmacwilliam@cs.harvard.edu>
#
#

require 'net/https'
require 'uri'
require 'optparse'

$GIT_BASE = "/var/www/html/git"
$PDF_BASE = "/var/www/html/pdf"
$URL_BASE = "repo-1.cs50.net"

trap("SIGINT") do
	`stty echo`
	puts ""
	exit!
end


course = ""
username = ""
project = ""

OptionParser.new do |opts|
	# course flag
	opts.on("-c", "--course [COURSE]", "Course") do |c|
		course = c
	end

	# username flag
	opts.on("-u", "--username [USERNAME]", "Git username") do |u|
		username = u
	end

	# repository flag
	opts.on("-r", "--repo [REPOSITORY]", "Project repository") do |r|
		project = r
	end
end.parse!

# path is requried argument
if (ARGV.length != 1)
	abort "Usage: submit /path/to/submit"
end
path = ARGV[0]

# prompt for course
if (course.empty?)
	print "Course: "
	course = STDIN.gets.chomp
end

# prompt for username
if (username.empty?)
	print "Username: "
	username = STDIN.gets.chomp
end

# prompt for project
if (project.empty?)
	print "Repository: "
	project = STDIN.gets.chomp
end

# prompt user for password, but don't output while typing
puts "Input your password below. For security, you won't see what you're typing."
`stty -echo`
print "Password: "
password = STDIN.gets.chomp
`stty echo`
puts ""

# change to directory user specified
Dir.chdir(path) do
	# create a new git repo if one doesn't already exist
	if (!File.directory?(".git"))
		puts "Creating git repository..."
		`git init`
	end

	# add and commit entire directory
	puts "Committing changes..."
	`git add --all`
	`git commit -m 'submit'`

	# tag commit as submission, replacing any previous submission
	puts "Tagging changes as submission..."
	`git tag submit --force`

	# push branch to server
	puts "Pushing changes..."
	`git push https://#{username}:#{password}@#{$URL_BASE}/git/#{course}/#{username}/#{project}.git master --quiet`

	# push failed as a result of non-fast-forward or invalid credentials
	if ($?.exitstatus != 0)
	    puts "Push rejected, cloning existing repository..."
	    # clone remote into a temporary directory
	    `rm -rf .git`
	    tmp_path = "/tmp/#{course}/#{username}/#{project}"
	    `mkdir -p #{tmp_path}`
	    `git clone https://#{username}:#{password}@#{$URL_BASE}/git/#{course}/#{username}/#{project}.git #{tmp_path} --quiet`
	    
	    # clone failed, so user must have supplied invalid credentials
	    if ($?.exitstatus != 0)
		abort "The username and password you supplied are incorrect, or you are trying submit to the wrong repository. Your project has NOT been submitted, please try again, double-checking your username, password, course, and project name."
	    end

	    # clone succeeded, so replace repo with new submit contents
	    puts "Clone successful, re-pushing..."
	    `rm -rf #{tmp_path}/*`
	    `cp -r * #{tmp_path}`

	    # submit from temp directory
	    Dir.chdir(tmp_path) do
		`git add --all`
		`git commit -m 'submit'`
		`git tag submit --force`
		`git push https://#{username}:#{password}@#{$URL_BASE}/git/#{course}/#{username}/#{project}.git master --quiet`
		`git push https://#{username}:#{password}@#{$URL_BASE}/git/#{course}/#{username}/#{project}.git --tags --quiet`
	    end

	    # move the temp .git directory into the working directory to avoid /tmp submission again
	    `cp -r #{tmp_path}/.git .`
	    `rm -rf #{tmp_path}`

	# push was successful, so proceed to push tags
	else
	    `git push https://#{username}:#{password}@#{$URL_BASE}/git/#{course}/#{username}/#{project}.git --tags --quiet`
	end

	puts "Changes pushed!"

	# check if PDF was rendered
	pdf_url = "https://#{$URL_BASE}/pdf/#{course}/#{username}/#{project}.git/submit.pdf"
	uri = URI.parse(pdf_url)

	# only show PDF url if PDF was actually rendered
	begin
		http = Net::HTTP.new(uri.host, uri.port)
		request = Net::HTTP::Get.new(uri.path)
		http.use_ssl = true
		http.verify_mode = OpenSSL::SSL::VERIFY_NONE
		request.basic_auth(username, password)
		response = http.request(request)
		response.value()
	rescue
		exit
	end

	puts "Your work has been submitted!"
	puts "To view your submission, visit https://#{$URL_BASE}/pdf/#{course}/#{username}/#{project}.git/submit.pdf"
end
